import { useMemo, useState } from "react";
import { useChat } from "@/context";
import { ChatType } from "@/types/chat";
import { ArrowPathIcon, ChevronRightIcon } from "@heroicons/react/24/solid";
import MarkdownRender from "./render-markdown";
import classNames from "classnames";

export default function BotMessageList() {
  const { chats, isBotReplying, bot } = useChat();

  const reveredChats = useMemo(() => [...chats].reverse(), [chats]);

  return (
    <div
      className="flex-1 flex flex-col-reverse gap-2 py-4 px-4 overflow-y-auto scrollbar"
      id="devi-chat-message-list"
    >
      {isBotReplying && <BotMessageBot message="Thinking..." />}
      {chats.length === 1 &&
        chats[0].id === "start" &&
        chats[0].afterFormSubmitted && (
          <FormCollection submit={chats[0].afterFormSubmitted} />
        )}
      {chats.length === 0 && (
        <div className="my-auto text-center mx-auto px-4 py-2.5 rounded-lg">
          <h2>
            <span className="font-bold text-2xl">{bot.name || "Bot"}</span>
          </h2>
          <p className="text-base mt-2">
            {bot.description ||
              "Hello, I will be your guide today. Please type something to start."}
          </p>
        </div>
      )}

      {reveredChats.map((chat) => {
        if (chat.role === "bot") {
          return <BotMessageBot key={chat.id} {...chat} />;
        } else if (chat.role === "user") {
          return <BotMessageUser key={chat.id} {...chat} />;
        } else {
          return null;
        }
      })}
    </div>
  );
}

function BotMessageBot({ message, isError }: Partial<ChatType>) {
  return (
    <div
      className={classNames(
        "py-1 px-3.5 text-xs rounded-xl inline-block w-auto max-w-[75%] mr-auto",
        {
          "bg-red-200 text-red-500": isError,
          "text-black bg-gray-200": !isError,
        }
      )}
    >
      <MarkdownRender content={message || ""} />
    </div>
  );
}

function BotMessageUser({ message }: ChatType) {
  return (
    <div className="py-1 px-3.5 text-xs rounded-xl inline-block w-auto max-w-[75%] ml-auto text-white bg-[var(--dc-primary)]">
      <MarkdownRender content={message || ""} />
    </div>
  );
}

function FormCollection({
  submit,
}: {
  submit: (formdata: { [key: string]: string }) => void;
}) {
  const { bot } = useChat();
  const [isSending, setisSending] = useState(false);

  return (
    <div className="flex flex-col gap-2 max-w-[75%] border px-3.5 py-3 rounded-xl">
      <p className="text-xs text-gray-500">
        {`This is the first time you're talking to this bot. Please fill the form below to continue.`}
      </p>

      <form
        className="flex flex-col gap-2"
        onSubmit={(e) => {
          e.preventDefault();

          const formData = new FormData(e.target as HTMLFormElement);
          const values: { [key: string]: string } = {};
          for (const [key, value] of formData.entries()) {
            values[key as keyof typeof values] = value as string;
          }

          setisSending(true);
          submit(values);
        }}
      >
        {Object.keys(bot.infoCollection?.start || {}).map((field) => {
          return (
            <div key={field} className="flex flex-col gap-1">
              <label htmlFor={field} className="text-xs text-gray-500">
                {field}
              </label>

              <input
                type={field?.toLowerCase().includes("email") ? "email" : "text"}
                name={field}
                id={field}
                required
                className="text-sm bg-gray-200 rounded-md px-2 py-1.5 outline-none"
              />
            </div>
          );
        })}

        <button
          className="bg-[var(--dc-primary)] text-white text-sm rounded-md px-2 py-1.5 w-max mt-2 flex items-center gap-2 outline-none disabled:opacity-50 disabled:cursor-not-allowed"
          type="submit"
          disabled={isSending}
        >
          <span>Continue</span>
          {isSending ? (
            <ArrowPathIcon className="h-4 w-4 animate-spin" />
          ) : (
            <ChevronRightIcon className="h-4 w-4" />
          )}
        </button>
      </form>
    </div>
  );
}
