import { ArrowPathIcon, PaperAirplaneIcon } from "@heroicons/react/24/solid";
import { useChat } from "@/context";
import { useEffect, useRef, useState } from "react";
import { chatApiRequest } from "@/lib/api";
import { APIError } from "@/lib/errors";
import { ChatType } from "@/types/chat";
import ConversationStarters from "./conversation-starters";

export default function BotSendMessage() {
  const { chats, bot, jwt, setstate, isBotReplying, setalert } = useChat();

  const chatInputBox = useRef<HTMLInputElement>(null);

  const [message, setmessage] = useState("");
  const [isSending, setisSending] = useState(false);
  const [currentSendingId, setcurrentSendingId] = useState({
    messageId: "",
    runId: "",
  });

  const scrollToBottom = () => {
    setTimeout(() => {
      const element = document.getElementById("devi-chat-message-list");
      if (element) {
        element.scrollTop = element.scrollHeight;
      }
    }, 50);
  };

  const send = async (msg: string = message) => {
    if (isSending) return;
    if (!msg) return;
    if (isBotReplying) return;

    if (
      chats.length === 0 &&
      bot.infoCollection?.start &&
      typeof bot.infoCollection.start === "object" &&
      !Array.isArray(bot.infoCollection.start) &&
      Object.keys(bot.infoCollection.start).length > 0
    ) {
      setstate("chats", [
        {
          id: "start",
          isSent: true,
          message: msg,
          role: "user",
          afterFormSubmitted: async (formdata) => {
            try {
              const { messageId, runId, formMessageId, formMessage } =
                await chatApiRequest(`/${bot._id}/thread/add-message`, {
                  message: msg,
                  jwt,
                  formdata,
                });

              setstate("chats", [
                {
                  id: messageId,
                  isSent: true,
                  message: msg,
                  role: "user",
                  sendAt: new Date(),
                } as ChatType,
                {
                  id: formMessageId,
                  isSent: true,
                  message: formMessage,
                  role: "user",
                  sendAt: new Date(),
                } as ChatType,
              ]);

              setstate("isBotReplying", true);
              setcurrentSendingId({ messageId, runId });
            } catch (error) {
              setalert({
                type: "ERROR",
                message: (error as APIError).message,
              });
            } finally {
              setisSending(false);
            }
          },
          sendAt: new Date(),
        } as ChatType,
      ]);

      setisSending(true);
      setmessage("");
      return;
    }

    try {
      setisSending(true);

      const { messageId, runId } = await chatApiRequest(
        `/${bot._id}/thread/add-message`,
        {
          message: msg,
          jwt,
        }
      );

      setmessage("");
      setcurrentSendingId({ messageId, runId });
      setstate("chats", [
        ...chats,
        {
          id: messageId,
          isSent: true,
          message: msg,
          role: "user",
          sendAt: new Date(),
        } as ChatType,
      ]);
      setstate("isBotReplying", true);
      scrollToBottom();
    } catch (error) {
      setalert({
        type: "ERROR",
        message: (error as APIError).message,
      });
    } finally {
      setisSending(false);
    }
  };

  useEffect(() => {
    if (!currentSendingId.runId) return;

    const interval = setInterval(async () => {
      try {
        const data = await chatApiRequest(`/${bot._id}/thread/get-messages`, {
          jwt,
          runId: currentSendingId.runId,
        });

        if (data.data.isFailed) {
          clearInterval(interval);
          setstate("chats", [
            ...chats,
            {
              id: data.data.messageId,
              isSent: true,
              message: data.data.message || `Something went wrong`,
              role: "bot",
              sendAt: new Date(),
              isError: true,
            } as ChatType,
          ]);
          setstate("isBotReplying", false);
          scrollToBottom();
        } else if (data.data.status === "completed") {
          clearInterval(interval);
          setstate("chats", [
            ...chats,
            {
              id: data.data.messageId,
              isSent: true,
              message: data.data.message,
              role: "bot",
              sendAt: new Date(),
            } as ChatType,
          ]);
          setstate("isBotReplying", false);
          scrollToBottom();
        }
      } catch (error) {
        setalert({
          type: "ERROR",
          message: (error as APIError).message,
        });
      }
    }, 2000);

    return () => clearInterval(interval);
  }, [currentSendingId]);

  useEffect(() => {
    if (chatInputBox) {
      chatInputBox.current?.focus();
    }
  }, []);

  return (
    <div>
      {chats.length === 0 && bot.conversationStarters.length > 0 && (
        <ConversationStarters
          onSelect={(text: string) => {
            send(text);
          }}
        />
      )}

      <div className="flex items-center bg-white border-t pb-1.5">
        <input
          ref={chatInputBox}
          type="text"
          className="w-full h-full py-4 outline-none px-4 text-base flex-1"
          placeholder="Type a message"
          value={message}
          onChange={(e) => setmessage(e.target.value)}
          onKeyDown={(e) => {
            if (e.key === "Enter") {
              send();
            }
          }}
        />

        <button
          className="text-[var(--dc-primary)] text-sm font-medium pr-4 pl-2 rounded-md disabled:opacity-50 disabled:cursor-not-allowed"
          onClick={() => send()}
          disabled={isSending || !message || isBotReplying}
        >
          {isSending ? (
            <ArrowPathIcon className="h-4 w-4 animate-spin" />
          ) : (
            <PaperAirplaneIcon className="h-4 w-4" />
          )}
        </button>
      </div>
    </div>
  );
}
