import { BotType } from "@/types/bot";
import { useEffect, useRef, useState } from "react";
import { ChatType } from "@/types/chat";
import { APIError } from "@/lib/errors";
import { chatApiRequest } from "@/lib/api";
import BotChatUi from "./components";
import ToggleButton from "./components/toggle-button";
import ChatUi, { ShowError, Skeleton } from "./components/chat-ui";

export default function InitChat({
  defaultOpen = false,
  botId,
  noToggleButton = false,
  usingTargetedDiv = false,
  chatOpenText = "",
}: {
  defaultOpen?: boolean;
  botId: string;
  noToggleButton?: boolean;
  chatOpenText?: string;
  usingTargetedDiv?: boolean;
}) {
  const [botIdLocal] = useState(botId);
  const [opened, setopened] = useState(defaultOpen);

  const [botAndJwt, setbotAndJwt] = useState<null | {
    inited: boolean;
    jwt?: string;
    error?: string | null;
    chats?: ChatType[];
    bot?: BotType;
  }>(null);

  const isIniting = useRef(false);

  async function init(force = false) {
    if (!opened) return;
    if (botAndJwt?.inited && !force) return;
    if (isIniting.current) return;

    try {
      isIniting.current = true;
      setbotAndJwt((prev) => ({ ...prev, inited: false }));

      const data = await chatApiRequest(`/${botIdLocal}/thread/create`, {
        jwt: localStorage.getItem("botjwt") || "",
      });

      setbotAndJwt({
        inited: true,
        jwt: data.jwt,
        chats: data.chats,
        bot: data.bot,
      });
      document.body.style.setProperty(
        "--dc-primary",
        data.bot?.cssColor || "#000000"
      );
      localStorage.setItem("botjwt", data.jwt);

      isIniting.current = false;
    } catch (error) {
      setbotAndJwt((prev) => ({
        jwt: prev?.jwt || "",
        inited: false,
        error: (error as APIError).message,
      }));
    }
  }

  useEffect(() => {
    init();

    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [opened]);

  return (
    <>
      {!noToggleButton && (
        <ToggleButton
          opened={opened}
          setopened={setopened}
          chatOpenText={chatOpenText}
        />
      )}

      {opened ? (
        <ChatUi fullWidthHeight={usingTargetedDiv}>
          {botAndJwt?.inited && botAndJwt.bot && botAndJwt.jwt ? (
            <BotChatUi
              bot={botAndJwt.bot}
              chats={botAndJwt.chats || []}
              jwt={botAndJwt.jwt}
              toggle={() => {
                if (noToggleButton && usingTargetedDiv) {
                  console.log(`Feature is disabled`);
                  return;
                }
                setopened((prev) => !prev);
              }}
              init={() => init(true)}
              usingTargetedDiv={usingTargetedDiv}
            />
          ) : botAndJwt?.error ? (
            <ShowError error={botAndJwt.error} setopened={setopened} />
          ) : (
            <Skeleton />
          )}
        </ChatUi>
      ) : null}
    </>
  );
}
